package com.dzeio.openhealth.ui.home

import android.animation.ValueAnimator
import android.graphics.BitmapFactory
import android.graphics.Canvas
import android.graphics.Rect
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.lifecycle.lifecycleScope
import androidx.navigation.fragment.findNavController
import androidx.preference.PreferenceManager
import com.dzeio.openhealth.Application
import com.dzeio.openhealth.R
import com.dzeio.openhealth.core.BaseFragment
import com.dzeio.openhealth.data.water.Water
import com.dzeio.openhealth.databinding.FragmentHomeBinding
import com.dzeio.openhealth.data.weight.Weight
import com.dzeio.openhealth.ui.weight.AddWeightDialog
import com.dzeio.openhealth.utils.BitmapUtils
import com.dzeio.openhealth.utils.DrawUtils
import com.github.mikephil.charting.components.AxisBase
import com.github.mikephil.charting.components.Description
import com.github.mikephil.charting.components.XAxis
import com.github.mikephil.charting.data.Entry
import com.github.mikephil.charting.data.LineData
import com.github.mikephil.charting.data.LineDataSet
import com.github.mikephil.charting.formatter.ValueFormatter
import com.google.android.material.color.MaterialColors
import dagger.hilt.android.AndroidEntryPoint
import kotlinx.coroutines.flow.*
import kotlinx.coroutines.launch
import java.text.DateFormat
import java.text.SimpleDateFormat
import java.util.*
import kotlin.math.min
import kotlin.properties.Delegates

@AndroidEntryPoint
class HomeFragment : BaseFragment<HomeViewModel, FragmentHomeBinding>(HomeViewModel::class.java) {

    companion object {
        const val TAG = "${Application.TAG}/HomeFragment"
    }

    override val bindingInflater: (LayoutInflater, ViewGroup?, Boolean) -> FragmentHomeBinding
        get() = FragmentHomeBinding::inflate

    private var intake by Delegates.notNull<Float>()

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        viewModel.init()

        binding.addWeight.setOnClickListener {
            AddWeightDialog().show(requireActivity().supportFragmentManager, null)
        }

        binding.fragmentHomeWaterAdd.setOnClickListener { _ ->

            val water = viewModel.water.value

            if (water == null) {

                val w = Water()
                w.value = 200
                viewModel.updateWater(w)
            } else {
                water.value += 200
                viewModel.updateWater(water)
            }
        }

        intake = PreferenceManager.getDefaultSharedPreferences(requireContext())
            .getString("water_intake", "1200")?.toFloat() ?: 1200f

        binding.fragmentHomeWaterTotal.text = "${intake.toInt()}ml"

        binding.fragmentHomeWaterRemove.setOnClickListener { _ ->

            val water = viewModel.water.value

            if (water != null) {

                water.value -= 200
                if (water.value == 0) {
                    viewModel.deleteWater(water)
                } else {
                    viewModel.updateWater(water)
                }
            }
        }

        binding.fragmentHomeWaterRemove.setOnClickListener {
            lifecycleScope.launch {

                val item = viewModel.fetchTodayWater().first()
                Log.d(TAG, "Collected latest $it")
                if (item != null) {
                    item.value -= 200
                    if (item.value == 0) {
                        viewModel.deleteWater(item)
                    } else {
                        viewModel.updateWater(item)
                    }
                }

            }
        }

        binding.listWeight.setOnClickListener {
            Log.d("T", "Trying to move")

            findNavController().navigate(HomeFragmentDirections.actionNavHomeToNavListWeight())
        }

        binding.gotoWaterHome.setOnClickListener {
            findNavController().navigate(HomeFragmentDirections.actionNavHomeToNavWaterHome())
        }
    }

    private fun updateGraph(list: List<Weight>) {
        if (list.isNotEmpty()) {
            val entries = ArrayList<Entry>()
            for (item in list) {
                entries.add(Entry(item.timestamp.toFloat(), item.weight))
            }

            val dataSet = LineDataSet(entries, "Label")
            binding.weightGraph.apply {

                // Setup
                isAutoScaleMinMaxEnabled = true
                legend.isEnabled = false
                isDragEnabled = true
                isScaleYEnabled = false
                description = Description().apply { isEnabled = false }
                isScaleXEnabled = true
                setPinchZoom(false)
                setDrawGridBackground(false)
                setDrawBorders(false)
                axisLeft.setLabelCount(0, true)

                xAxis.apply {
                    valueFormatter = object : ValueFormatter() {
                        override fun getAxisLabel(value: Float, axis: AxisBase?): String {
                            return SimpleDateFormat(
                                "yyyy-MM-dd",
                                Locale.getDefault()
                            ).format(Date(value.toLong()))
                            //return super.getAxisLabel(value, axis)
                        }
                    }
                    position = XAxis.XAxisPosition.BOTTOM
                    setDrawGridLines(false)
                    setLabelCount(3, true)
                }

                // Apply new dataset
                data = LineData(dataSet)

                // idk what I did but it works lol
                setVisibleXRange(
                    0f, entries[entries.size - 1].x / 1000f
                )

                // BIS... :(
                // Also it invalidate the view so I don't have to call invalidate
                moveViewToX(entries[entries.size - 1].x - 1600000000f)
            }
        }
    }

    override fun onStart() {
        super.onStart()

        lifecycleScope.launchWhenStarted {
            viewModel.fetchWeights().collectLatest {
                updateGraph(it)
            }
        }

        viewModel.water.observe(viewLifecycleOwner) {
            Log.d(TAG, "${it?.formatTimestamp()} $it")
            if (it != null) {
                updateWater(it.value)
            } else {
                updateWater(0)
            }
        }

        updateWater(0)

    }

    private fun updateWater(water: Int) {
        val oldValue = binding.fragmentHomeWaterCurrent.text.toString().replace("ml", "").toInt()
        binding.fragmentHomeWaterCurrent.text = "${water}ml"

        val graph = BitmapUtils.convertToMutable(
            requireContext(),
            BitmapFactory.decodeResource(resources, R.drawable.ellipse)
        )

        graph?.let { btmp ->
            ValueAnimator.ofFloat(min(oldValue.toFloat(), intake), min(water.toFloat(), intake))
                .apply {
                    duration = 300
                    addUpdateListener {
                        val canvas = Canvas(btmp)
                        DrawUtils.drawArc(
                            canvas,
                            100 * it.animatedValue as Float / intake,
                            MaterialColors.getColor(
                                requireView(),
                                com.google.android.material.R.attr.colorPrimary
                            )
                        )
                        canvas.save()
                        binding.background.setImageBitmap(graph)
                    }
                    start()
                }
        }
    }
}